@@ -1,16 +1,18 @@
 import ast
+import collections
 import configparser
 import itertools
 import logging
 from importlib.metadata import EntryPoint, entry_points
 from pathlib import Path
-from typing import Iterator, Optional
+from typing import Iterable, Iterator, Optional
 
 log = logging.getLogger(__name__)
 
 
 def scan_entrypoints(
     group_name: str,
+    allowlist: Optional[set[str]] = None,
     blocklist: Optional[set[str]] = None,
 ) -> Iterator[EntryPoint]:
     if blocklist is None:
@@ -21,7 +23,9 @@ def scan_entrypoints(
         scan_entrypoint_from_plugin_checkouts(group_name),
         scan_entrypoint_from_package_metadata(group_name),
     ):
-        if match_blocklist(entrypoint.value, blocklist):
+        if allowlist is not None and not match_plugin_list(entrypoint.value, allowlist):
+            continue
+        if match_plugin_list(entrypoint.value, blocklist):
             continue
         if existing_entrypoint := existing_names.get(entrypoint.name, None):
             if existing_entrypoint.value == entrypoint.value:
@@ -43,14 +47,14 @@ def scan_entrypoints(
         yield entrypoint
 
 
-def match_blocklist(entry_path: str, blocklist: set[str]) -> bool:
+def match_plugin_list(entry_path: str, plugin_list: set[str]) -> bool:
     """
-    Checks if the given module attribute reference is in the blocklist.
-    The blocklist items are assumeed to be prefixes of package import paths
+    Checks if the given module attribute reference is in the plugin_list.
+    The plugin_list items are assumeed to be prefixes of package import paths
     or the package namespaces.
     """
     mod_path = entry_path.partition(":")[0]
-    for block_pattern in blocklist:
+    for block_pattern in plugin_list:
         if mod_path.startswith(block_pattern + ".") or mod_path == block_pattern:
             return True
     return False
@@ -60,6 +64,34 @@ def scan_entrypoint_from_package_metadata(group_name: str) -> Iterator[EntryPoin
     yield from entry_points().select(group=group_name)
 
 
+_default_glob_excluded_patterns = [
+    "ai/backend/webui",
+    "ai/backend/web/static",
+    "ai/backend/runner",
+    "ai/backend/kernel",
+]
+
+
+def _glob(base_path: Path, filename: str, excluded_patterns: Iterable[str]) -> Iterator[Path]:
+    q: collections.deque[Path] = collections.deque()
+    q.append(base_path)
+    while q:
+        search_path = q.pop()
+        assert search_path.is_dir()
+        for item in search_path.iterdir():
+            if item.is_dir():
+                if search_path.name == "__pycache__":
+                    continue
+                if search_path.name.startswith("."):
+                    continue
+                if any(search_path.match(pattern) for pattern in excluded_patterns):
+                    continue
+                q.append(item)
+            else:
+                if item.name == filename:
+                    yield item
+
+
 def scan_entrypoint_from_buildscript(group_name: str) -> Iterator[EntryPoint]:
     entrypoints = {}
     # Scan self-exported entrypoints when executed via pex.
@@ -67,7 +99,7 @@ def scan_entrypoint_from_buildscript(group_name: str) -> Iterator[EntryPoint]:
     log.debug(
         "scan_entrypoint_from_buildscript(%r): Namespace path: %s", group_name, ai_backend_ns_path
     )
-    for buildscript_path in ai_backend_ns_path.glob("**/BUILD"):
+    for buildscript_path in _glob(ai_backend_ns_path, "BUILD", _default_glob_excluded_patterns):
         for entrypoint in extract_entrypoints_from_buildscript(group_name, buildscript_path):
             entrypoints[entrypoint.name] = entrypoint
     # Override with the entrypoints found in the current source directories,
@@ -78,7 +110,7 @@ def scan_entrypoint_from_buildscript(group_name: str) -> Iterator[EntryPoint]:
     else:
         src_path = build_root / "src"
         log.debug("scan_entrypoint_from_buildscript(%r): current src: %s", group_name, src_path)
-        for buildscript_path in src_path.glob("**/BUILD"):
+        for buildscript_path in _glob(src_path, "BUILD", _default_glob_excluded_patterns):
             for entrypoint in extract_entrypoints_from_buildscript(group_name, buildscript_path):
                 entrypoints[entrypoint.name] = entrypoint
     yield from entrypoints.values()
@@ -98,11 +130,11 @@ def scan_entrypoint_from_plugin_checkouts(group_name: str) -> Iterator[EntryPoin
             plugins_path,
         )
         # For cases when plugins use Pants
-        for buildscript_path in plugins_path.glob("**/BUILD"):
+        for buildscript_path in _glob(plugins_path, "BUILD", _default_glob_excluded_patterns):
             for entrypoint in extract_entrypoints_from_buildscript(group_name, buildscript_path):
                 entrypoints[entrypoint.name] = entrypoint
         # For cases when plugins use standard setup.cfg
-        for setup_cfg_path in plugins_path.glob("**/setup.cfg"):
+        for setup_cfg_path in _glob(plugins_path, "setup.cfg", _default_glob_excluded_patterns):
             for entrypoint in extract_entrypoints_from_setup_cfg(group_name, setup_cfg_path):
                 if entrypoint.name not in entrypoints:
                     entrypoints[entrypoint.name] = entrypoint
